local waitlist = {}
local responsePrefix = "<Waitlist> "
local responseFilter = "^"..responsePrefix
local commandFilter = "^wl "
local outputPrefix = "|cffffff7fWaitlist:|r "
local outputFrame = 1
local verbose = false
local squelch = true
local altPrefix = "*"
local altSuffix = "*"

local function getmain(alt)
	local main = nil
	for i = 1, GetNumGuildMembers() do
		local name, _, _, _, _, _, note = GetGuildRosterInfo(i)
		if name == alt and note:find(altPrefix.."%a"..altSuffix) then
			main = string.sub(note, (string.len(altPrefix))+1, -(string.len(altSuffix)+1))
			break
		end
	end	
	return main
end

local function addname(name)
	local message = nil
	if UnitInRaid(name) then
		message = name.." is already in the raid"
	else
		for k,v in ipairs(waitlist) do
			if v == name then
					message = name.." is already on the waitlist"
				break
			end
		end
	end
	if not message then
		table.insert(waitlist, name)
		message = name.." has been added to the waitlist"
	end
	return message
end

local function clobber(name)
	local message = nil
	for k,v in ipairs(waitlist) do
		if v == name then
			table.remove(waitlist, k)
			message = name.." has been removed from the waitlist"
			break
		end
	end
	return message
end

local function listnames()
	local message = nil
	if table.getn(waitlist) == 0 then
		message = "No one is on the wait list" 
	else
		message = "Current Waitlist: "
		for k,v in ipairs(waitlist) do
			message = message..v..", "
		end
		message = string.sub(message, 1, -3)
	end
	return message
end

local function fixcase(small)
	return string.gsub(string.lower(small), "%a", string.upper, 1) 
end

local function outgoingFilter()
	if arg1:find(responseFilter) then return squelch end
end

local function incomingFilter()
	if arg1:find(commandFilter) then 
		local response = nil
		local input = string.lower(arg1)
		if input:find("add") then
			response = addname(arg2)
			if verbose then _G["ChatFrame"..outputFrame]:AddMessage(outputPrefix..response) end
		elseif input:find("main") then
			local main = getmain(arg2)
			if main then
				response = addname(main)
				if verbose then _G["ChatFrame"..outputFrame]:AddMessage(outputPrefix..response) end
			else
				response = "Unable to find main name for "..arg2.." on the guild roster"
			end
		elseif input:find("del") or input:find("remove") then
			response = clobber(arg2)
		elseif input:find("who") or input:find("list") or input:find("report") then
			response = listnames()
		else
			response = "Usage: whisper me 'wl add', 'wl main', 'wl del', 'wl remove', 'wl who', 'wl list', or 'wl report'"
		end
		if response then SendChatMessage(responsePrefix..response, "WHISPER", nil, arg2) end
		return squelch
	end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", outgoingFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", incomingFilter)

local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function(self, event, msg)
	if event == "RAID_ROSTER_UPDATE" then
		for i = 1, GetNumRaidMembers() do
			clobber(UnitName("raid"..i))
		end
	end
end)

frame:RegisterEvent("RAID_ROSTER_UPDATE")

SLASH_WAITLIST1 = "/waitlist";
SLASH_WAITLIST2 = "/wl";
SlashCmdList["WAITLIST"] = 
   function(msg)
   	local output
	local input = string.lower(msg)
	if input:find("^add ") then
		output = addname(fixcase(string.trim(string.sub(input, string.len("add ")))))
	elseif input:find("^del ") then
		output = clobber(fixcase(string.trim(string.sub(input, string.len("del ")))))
	elseif input:find("^remove ") then
		output = clobber(fixcase(string.trim(string.sub(input, string.len("remove ")))))
	elseif input:find("^who") or input:find("^list")  or input:find("^report") then
		output = listnames()
		if input:find("officer") then
			SendChatMessage(responsePrefix..output, "OFFICER")
			output = nil
		elseif input:find("guild") then
			SendChatMessage(responsePrefix..output, "GUILD")
			output = nil
		elseif input:find("raid") then
			SendChatMessage(responsePrefix..output, "RAID")
			output = nil
		elseif input:find("party") then
			SendChatMessage(responsePrefix..output, "PARTY")
			output = nil
		elseif input:find("say") then
			SendChatMessage(responsePrefix..output, "SAY")
			output = nil
		end
	elseif input:find("^reset") or input:find("^clear") then
		waitlist = {}
		output = "Waitlist has been reset"
	else
		output = "Usage: /wl | /waitlist add | del/remove | who/list/report | reset/clear"
	end
	if output then
		_G["ChatFrame"..outputFrame]:AddMessage(outputPrefix..output)
	end
   end
